#!/bin/sh
set -a

clear

# Check that required files exist within the package
echo "\nChecking required files to apply patch... \c"
for fl in WEB3.6FP3.UpgradeList WEB3.6FP3.tar
do
	if [ ! -f $fl ]
	then
		echo "FAILED !!.\n\tRequired '$fl' is missing from the tarball.\n"
		exit
	fi
done
echo "PASSED."

# Check that the the WEB installation directory is defined 
echo "\nChecking for a valid Metrica/WEB installation directory (\$NPRWEB_DIR)... \c"
if [ ! -f $NPRWEB_DIR/cgi/webnpr3 ]
then
	echo "FAILED !! .\n\tMissing '$NPRWEB_DIR/cgi/webnpr3'. Please check the installation.\n"
	exit
fi
echo "PASSED."

# Locating files that need to be changed
echo "\nLocating files..."
for fl in `cat WEB3.6FP3.UpgradeList`
do
	echo "\t $NPRWEB_DIR/$fl... \c"
	if [ ! -f $NPRWEB_DIR/$fl ]
	then
		echo "FAILED !!\n"
		exit
	else
		echo "PASSED."
	fi
done
echo "ALL FOUND."

currdir=`pwd`

# Create backup for all files that need to be changed
echo "\nBacking up files as... \c"
cd $NPRWEB_DIR
for fl in `cat $currdir/WEB3.6FP3.UpgradeList`
do
	mv $fl $fl.preFP3
done
echo "DONE."

echo "\nApplying patch WEB3.6FP3.tar... \c"
tar xpf $currdir/WEB3.6FP3.tar
echo "COMPLETED.\n"

# Finally go back to patching directory
cd $currdir
