# ************************************************************
# @(#) web/webserver/cgi/Core/Admin/conf_display.pm web_r3.6.0.8:cset.001111:22:33@(#)
#
#   Filename    : web/webserver/cgi/Core/Admin/conf_display.pm
#
# *************************************************************
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                       Vallent
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# Copyright (c) 2005 Vallent Software Systems UK. All rights reserved.
#
# **************************************************************
#
#   Description : configure class for creating controlling the
#                 configure servers page.
#
#
# ************************************************************
use strict;

package Core::Admin::conf_display;

use vars qw(@ISA);

require Core::Configuration;

@ISA = qw(Exporter Core::Configuration);

#--------------------------------------------------------------

# set up logging
require "$ENV{'NPRWEB_DIR'}/cgi/Lib/Logger.pm";
my $logger = new Logger( $ENV{NPRWEB_NPR_LOGFILE}, $ENV{NPRWEB_NPR_LOGLEVEL} );

###################################################################
# Subroutine name:      new()
#
# Description:          create an instance of the class conf_display
#			displays the pages to administrate the 
#			network elements and data items
#
#
sub new {
    my $proto = shift;    # get the class name
    my $class = ref($proto) || $proto;
    my $this = {MW_system => {},
                @_};

    bless ($this, $class);
    return $this ;
}

###################################################################
# Subroutine name:      display_function()
#
# Description:          calls the appropriate functions to build the
#  			HTML page, and display it.
#
#
sub display_function {
    my $this = shift;
    my $type = $this->{MW_system}{TYPE};

    $logger->log("info - Entering display_function", 4); 

    my $len;
    my $ad_html="";

    my $title="Administration of NPR Data Model";
    my $param_chk_file= $this->{MW_system}{CGIPROG};
    my $add=1;

    #--------------------------------------------------------------


    # Establish the type of HTML page first
    $ad_html .= "<HEAD>\n";
    $ad_html .= "<TITLE>$title</TITLE>\n";
    $ad_html .= "<LINK REL=\'stylesheet\' HREF=\'/css/style.css\' TYPE=\'text/css\'>\n";
    $ad_html .= "<SCRIPT LANGUAGE=\"JavaScript\" SRC=\"/javascript/preload.js\"></SCRIPT>\n";
    $ad_html .= "</HEAD>\n";
    $ad_html .= "<BODY background=\'/images/framebg.gif\' onload=\"MM_preloadImages('RegenQueriesRulesOveri.gif','RegenCacheOver.gif','addNewRecordOver.gif','removeSelectedRecordsOver.gif','ResetOver.gif')\">\n\n";

    $ad_html .= $this->_display_NE_header() ;

    $ad_html .="\n<center><FORM name=nprform action=$param_chk_file method=post>\n";
    $ad_html .="<input type=hidden name=NPRTARGET value=admin/process_param.htm>\n";
    $ad_html .="<input type=hidden name=NPRSYS value=$this->{MW_system}{NPRSYS}>\n";
    $ad_html .="<input type=hidden name=TYPE value=$type>\n";
    $ad_html .="<input type=hidden name=ne_di value=''>\n";
    $ad_html .="<input type=hidden name='WIZ_LIST' value=''>\n";


    if ($type eq 'ne'){
         $this->{dataBaseItems} = $this->readDATABASE("$this->{MW_system}{MW_NPRWEB_TECH}/database.cfg") ;
	 $len = $this->{dataBaseItems};
	 $len= keys( %$len);
         $ad_html .= $this->_build_NE_table(), if ($type eq 'ne');
         $ad_html .= $this->_read_ne_file();
    }
    elsif ($type eq 'di'){
         $ad_html .= "<CENTER><p class=\"bluehead0\">Add/Remove/Modify Data Items</p></CENTER><HR>";
         $this->{NPRVars} = $this->readNPRVARS("$this->{MW_system}{MW_NPRWEB_TECH}/nprvars.cfg") ;
    	 $len = $this->{NPRVars};
         $len= keys( %$len);
         $ad_html .= $this->_build_DI_table();
         $ad_html .= $this->_read_di_file();
    }
    elsif ($type eq 'hidden'){
         $ad_html .= "<CENTER><p class=\"bluehead0\">Add/Remove/Modify Hidden Items</p></CENTER><HR>";
         $this->{NPRVarsHidden} = $this->readNPRVARS_HIDDEN("$this->{MW_system}{MW_NPRWEB_TECH}/nprvars_hidden.cfg") ;
         $len = $this->{NPRVarsHidden};
         $len= keys( %$len);
         $ad_html .= $this->_build_Hidden_table();
         $ad_html .= $this->_read_Hidden_file();
    }
    else{
         $logger->log("Error - Admin::conf_display:  unknown type $type", 1);
         die("1000; Admin::conf_display:  unknown type $type");
    } 

    $ad_html .= $this->_close_table();
    $ad_html .= "<center><p class=\"bluehead4\">Total of $len record(s)</p></center>\n";
    $ad_html .= $this->_close_html_page();

    $logger->log("info - Leaving display_function", 4); 
    return $ad_html;
}

################################################################################
# Subroutine name:  _display_NE_header()
#
# Description:
#   This function displays the header for the Network Element dialog
#
# Return TRUE
#
sub _display_NE_header{
    my ($this) = @_ ;
    my $html = "";
    if ($this->{MW_system}{TYPE} eq 'ne') {
       $html .= "<CENTER><p class=\"bluehead0\">Queries/Rules, Cache and Network Elements</p></CENTER>\n";
   
       $html .= qq[<hr>\n<form name=cache_queries action="$this->{MW_system}{CGIPROG}">\n
                   <center>
		   <input type=hidden name=NPRTARGET value="admin/cache.htm">
                   <input type=hidden name=NPRSYS value=$this->{MW_system}{NPRSYS}>
                   <input type=hidden name=TL value=$this->{MW_system}{TECH_LAYER}>
                   <!--create input CACHE to be populated by Cache() on submition-->
                   <input type=hidden name=CACHE value="">\n
                   <input type=image src="/images/buttons/RegenQueriesRules.gif" title="click to regenerate Queries/Rules" id="regenQueries" onclick="return QR()" onmouseover="MM_swapImage('regenQueries','','/images/buttons/RegenQueriesRulesOver.gif',1)" onmouseout="MM_swapImgRestore()">
                   <input type=image src="/images/buttons/RegenCache.gif" title="click to regenerate the cache" id="regenCache" onclick="return Cache()" onmouseover="MM_swapImage('regenCache','','/images/buttons/RegenCacheOver.gif',1)" onmouseout="MM_swapImgRestore()">\n
                </center></form><hr>
]; 
    }
    return $html ;
}


################################################################################
# Subroutine name:  _build_NE_table()
#
# Description:
#   This function build the headings for the Network Element table
#
# Return TRUE
#
sub _build_NE_table{
     my $this = shift;
     # Build the basic table with column names
     my $html =qq[<table border=1 width=100% height=47><tr>
     <td class="tableblueboxes" width=28% >Display Label</td>
     <td class="tableblueboxes" width=20% >Item Name</td>
     <td class="tableblueboxes" width=50% >Elements selected by</td>
     <td class="tableblueboxes" width=2%  >Remove?</td>
         </tr>];

        return $html;
}

################################################################################
# Subroutine name:  _build_DI_table()
#
# Description:
#   This function build the headings for the Data Items table
#
# Return TRUE
#
sub _build_DI_table{
     my $this = shift;
     # Build the basic table with column names
     my $html =qq[<table border=1 width=80% height=47><tr>
     <td class="tableblueboxes" width=50% >Display Label</td>
     <td class="tableblueboxes" width=20% >Item Name</td>
     <td class="tableblueboxes" width=2%  >Remove?</td>
         </tr>];

        return $html;
}

################################################################################
# Subroutine name:  _build_Hidden_table()
#
# Description:
#   This function build the headings for the Data Items table
#
# Return TRUE
#
sub _build_Hidden_table{
     my $this = shift;
     # Build the basic table with column names
     my $html =qq[<table border=1 width=80% height=47><tr>
     <td class="tableblueboxes" width=50% >Display Label</td>
     <td class="tableblueboxes" width=20% >Item Value</td>
     <td class="tableblueboxes" width=2%  >Remove?</td>
         </tr>];

        return $html;
}


################################################################################
# Subroutine name:  _read_ne_file()
#
# Description:
#  The database.cfg file is read, and used to populate the table
#
# Arguments:
#            in_file  (scalar) - Filename and Path
#            url_file (scalar) - Filename and Path, URL encoded
#
# Return TRUE
# Returns (on HTML submit):
#            TYPE     - NE
#            FILENAME - direcotry path and filename
#            (list of Network Elements) - TRUE or FALSE (for removal)
#
sub _read_ne_file {
    my $this = shift;
    my $html;

    my $dataBaseItems = $this->{dataBaseItems};

    #build the table from each key
    my $DBItem;
    my ($a, $d, $g);
    foreach ( sort keys %$dataBaseItems ) {
        $a= $$dataBaseItems{$_}{Name};
        $d= $$dataBaseItems{$_}{DisplayName};
        $g= $$dataBaseItems{$_}{SelectBy};
$logger->log("$a :: $d :: $g :: $$dataBaseItems{$_}{Name}", 1);

   	$html .= "\n<tr>\n";
	$html .= qq[<td class="tableblueboxes1"><A HREF='#' onclick=" window.open('$this->{MW_system}{CGIPROG}?NPRSYS=$this->{MW_system}{NPRSYS}&NPRTARGET=admin/modify_ne_di.htm&ne_di=ne&add=0&data=$_','modify_ne_$this->{MW_system}{NPRSYS}','resizable,height=650,width=650,scrollbar')">$d</A> 
          </td>
         <td class="tableblueboxes1">$a</td>
         <td class="tableblueboxes1">@$g</td>
         <td class="tableblueboxes1"><div align=center><center>
                 <p><input type=checkbox name=$a value=REMOVE></p>
         </center></div></td></tr>];

    }
    $html .= "<input type=hidden name=TYPE value=ne>";
    return $html;
}

################################################################################
# Subroutine name:  read_di_file()
#
# Description:
#  The in_file_di file is read, and used to populate the table
#
# Arguments:
#            in_file  (scalar) - Filename and Path
#            url_file (scalar) - Filename and Path, URL encoded
#
# Return TRUE
# Returns (on HTML submit):
#            TYPE     - DI
#            FILENAME - direcotry path and filename
#            (list of data items) - TRUE or FALSE (for removal)
#
sub _read_di_file {
    my $this = shift;
    my $html;
    my ($a, $b);
    my $NPRVars = $this->{NPRVars};

    foreach ( sort keys %$NPRVars ) {
        $a = $$NPRVars{$_}{Name};
	$b = $$NPRVars{$_}{DisplayName};

        $html .= qq[
	 <tr>
         <td class=\"tableblueboxes1\"><A HREF='#' onclick=window.open('$this->{MW_system}{CGIPROG}?NPRSYS=$this->{MW_system}{NPRSYS}&NPRTARGET=admin/modify_ne_di.htm&ne_di=di&add=0&in_file=&data=$_','modify_di_$this->{MW_system}{NPRSYS}','resizable,height=650,width=650,scrollbar')>$b</A>
                         </td>
         <td class=\"tableblueboxes1\">$a</td>
         <td class=\"tableblueboxes1\"><div align=center><center>
                 <p><input type=checkbox name=$a value=REMOVE></p>
        ];
    }
    return $html; 
}

################################################################################
# Subroutine name:  read_Hidden_file()
#
# Description:
#  The nprvars_hidden.cfg file is read, and used to populate the table
#
# Arguments:
#            in_file  (scalar) - Filename and Path
#            url_file (scalar) - Filename and Path, URL encoded
#
# Return TRUE
# Returns (on HTML submit):
#            TYPE     - DI
#            FILENAME - direcotry path and filename
#            (list of data items) - TRUE or FALSE (for removal)
#
sub _read_Hidden_file {
    my $this = shift;
    my $html;
    my ($a, $b);
    my $NPRVarsHidden = $this->{NPRVarsHidden};

    foreach ( sort keys %$NPRVarsHidden ) {
        $a = $$NPRVarsHidden{$_}{Name};
        $b = $$NPRVarsHidden{$_}{Value};

        $html .= qq[
         <tr>
         <td class=\"tableblueboxes1\"><A HREF='#' onclick=window.open('$this->{MW_system}{CGIPROG}?NPRSYS=$this->{MW_system}{NPRSYS}&NPRTARGET=admin/modify_ne_di.htm&ne_di=hidden&add=0&data=$_','modify_di_$this->{MW_system}{NPRSYS}','resizable,height=650,width=650,scrollbar')>$b</A>
                         </td>
         <td class=\"tableblueboxes1\">$a</td>
         <td class=\"tableblueboxes1\"><div align=center><center>
                 <p><input type=checkbox name=$a value=REMOVE></p>
        ];
    }
    return $html;
}

################################################################################
# Subroutine name:  close_table()
#
# Description:
#  Closes the HTML table
#
# Return TRUE
#
sub _close_table{
        my $html='</TABLE></TD></TR></TABLE><BR CLEAR=ALL>';
        return $html;
}

################################################################################
# Subroutine name:  _close_html_page()
#
# Description:
#  This functioning closes the HTML page, and applies JavaScript
#  to confirm and items selected for REMOVAL.
#
# Arguments:
#
# Return (on HTML submit): TRUE or FALSE
#
sub _close_html_page{
        my $this = shift;
        my $end_html = "";
        # Java Script used to alert user of remove selection
        # Check all elements on the form. If the value is set to
        # Remove, then display this in a confirm box.
        $end_html .= qq[<script  language=javascript>
                function QR(){
                        return confirm("Are you sure you wish to rebuild the query/rule files?");
                }

                function Cache(){
                        //populate hidden input CACHE so the cache gets generated
                        document.cache_queries.CACHE.value = 'CACHE';
                        return confirm("Are you sure you wish to rebuild the cache files?");
                }
                function Remove_alert(){
                        // Check form elements in array
                        // If any radios with the value REMOVE are checked to TRUE
                        // Add these to a warning dialogue, else return TRUE
                        var msg="";

                        for( var i = 0; i < document.nprform.elements.length; i++){

                                if((document.nprform.elements[i].checked)&&(document.nprform.elements[i].value == 'REMOVE')){
                                        msg += document.nprform.elements[i].name+"\\n";
                                }

                        }

                        // User confirm with either OK or CANCEL
                        // If OK return TRUE else return FALSE.
                        if (msg){

                                // If this is NE section
                                if (document.nprform.TYPE.value == 'ne' ){
                                alert("If you remove a Network Element you may need to rebuild the queries/rules and Cache");
                                }

                                return confirm("Are you sure you wish to remove the following items?\\n\\n"+msg);}
                        alert("You have not selected anything to remove");
                        return false;
                }
</script>
        ];

        $end_html .= qq[
                <input type=hidden name=goto value='process_ne_di.pm' >
                </FORM>
                <p>
        ];

        
        $end_html .= qq[
                 <center>
                 <input type=image src='/images/buttons/addNewRecord.gif' title='Add new record' id='add_rec' onClick="window.open('$this->{MW_system}{CGIPROG}?NPRSYS=$this->{MW_system}{NPRSYS}&NPRTARGET=admin/modify_ne_di.htm&ne_di=$this->{MW_system}{TYPE}&add=1','modify_$this->{MW_system}{NPRSYS}','resizable,height=650,width=650,scrollbar')" name=add_record onmouseover="MM_swapImage('add_rec','','/images/buttons/addNewRecordOver.gif',1)" onmouseout="MM_swapImgRestore()">
        ];

        $end_html .= qq[
                &nbsp;&nbsp;&nbsp;
                <input type=image src='/images/buttons/removeSelectedRecords.gif' title='Remove selected record(s)' id='remove' onclick="
                var ret=Remove_alert();
                if( ret ){
                     document.nprform.ne_di.value='$this->{MW_system}{TYPE}_remove'
                     document.nprform.submit();
                }
                "
                onmouseover="MM_swapImage('remove','','/images/buttons/removeSelectedRecordsOver.gif',1)" onmouseout="MM_swapImgRestore()">
                &nbsp;&nbsp;&nbsp;
                <input type=image src='/images/buttons/Reset.gif' name=b2 id='Reset' onclick='document.nprform.reset();' onmouseover="MM_swapImage('Reset','','/images/buttons/ResetOver.gif',1)" onmouseout="MM_swapImgRestore()"></center></P>
        </center>
        </BODY>];


    return $end_html;
}



1;
