# ************************************************************
# @(#) web/PAS/cgi/Core/Form/Alarm.pm web_r3.6.0.8:cset.001111:22:33@(#)
#
#   Filename    : /webserver/cgi/Core/Form/Alarm.pm
#
# *************************************************************
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                       Vallent
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# Copyright (c) 2005 Vallent Software Systems UK. All rights reserved.
#
# **************************************************************
#
#   Description :
#
# ************************************************************
#
package Core::Form::Alarm ;

require Core::Form ;

require Core::Form::KeyUtils;

@ISA = qw{ Core::Form Core::Form::KeyUtils };

use Date::Manip ;
use Fcntl;
use FileHandle;

# set up logging
require "$ENV{'NPRWEB_DIR'}/cgi/Lib/Logger.pm";
my $logger = new Logger( $ENV{NPRWEB_NPR_LOGFILE}, $ENV{NPRWEB_NPR_LOGLEVEL} );

##################################################################################
#
# Function    : new
#
# Parameters  : MW_system - hash of form variables,
#
# Description : Constructor for reports/graphs Form
#
# Returns     : object reference
#
# Called By   : build.htm
#
##################################################################################
sub new {
    my $proto = shift;    # get the class name
    my $class = ref($proto) || $proto;
    # if class exists, use it else use reference.
    my $this = { MW_system => {},
                 MW_formdata => {},
                              @_ } ;

    # Set default values or store form data in environment variables
    if ($this->{MW_system}{'PREFERENCEA'} eq "") {
        $this->{MW_system}{'PREFERENCEA'} = &main::default_preferences() ;
	$this->{MW_system}{'REFRESHRATE'} = '60';
    } else {
        if ($this->{MW_formdata}{critical} ne "" ) {
		$this->{MW_system}{'PREFERENCEA'} = "critical,$this->{MW_formdata}{critical},";
		$this->{MW_system}{'PREFERENCEA'} .= "major,$this->{MW_formdata}{major},";
		$this->{MW_system}{'PREFERENCEA'} .= "minor,$this->{MW_formdata}{minor},";
                $this->{MW_system}{'PREFERENCEA'} .= "warning,$this->{MW_formdata}{warning},";
		$this->{MW_system}{'PREFERENCEA'} .= "indeterminate,$this->{MW_formdata}{indeterminate},";
		$this->{MW_system}{'PREFERENCEA'} .= "cleared,$this->{MW_formdata}{cleared},";
		$this->{MW_system}{'REFRESHRATE'} = $this->{MW_formdata}{rate};
        }
    }

    if ($this->{MW_system}{'SORTA'} eq '') {
	$this->{MW_system}{'SORTA'} = '6,ASC,,3,ASC,,0,DESC,,1,DESC,';
    } else {
	if ($this->{MW_formdata}{sortOrder} ne "" ) {
		$this->{MW_system}{'SORTA'} = $this->{MW_formdata}{sortOrder};
	}
    }

    if ($this->{MW_system}{'FIELDA'} eq "") {
        $this->{MW_system}{'FIELDA'} = &main::default_fields() ;
    } else {
	if ($this->{MW_formdata}{fieldsOrder} ne "" ) {
        	$this->{MW_system}{'FIELDA'} = $this->{MW_formdata}{fieldsOrder};
	}
    }

    if ($this->{MW_system}{'FILTERA'} eq "") {
        $this->{MW_system}{'FILTERA'} = &main::default_filter() ;
    } else {
        if ($this->{MW_formdata}{'filterOrder'} ne "" ) {
                $this->{MW_system}{'FILTERA'} = $this->{MW_formdata}{'filterOrder'};
        }
    }

    $logger->log("filterOrder = $this->{MW_formdata}{'filterOrder'}",1);

    if ($this->{MW_system}{'AGEA'} eq "") {
        $this->{MW_system}{'AGEA'} = '60';
    } else {
        if ($this->{MW_formdata}{filterAge} ne "" ) {
                $this->{MW_system}{'AGEA'} = "$this->{MW_formdata}{filterAge}";
        }
    }

    # alarm information
    $this->{alarmfile} = "$this->{MW_system}{'MW_NPRWEB_NPR_HOME'}/tmp/avalarms" ;

    # display alarm data
#    $this->{MW_system}{alarmHeader} = [];
#    $this->{MW_system}{displayAlarm} = [];
#    $this->{MW_system}{initNumAlarms} = ();

    # Description : define routines for each sort option
    $this->{sortType} = {

        0 => sub { my( $x, $y, $dir ) = @_; return $dir * ( $x cmp $y ) },   # Object Class
        1 => sub { my( $x, $y, $dir ) = @_; return $dir * ( $x cmp $y ) },   # Object Name
        3 => sub {                                                           # Event Time
            my ( $x, $y, $dir ) = @_;
            $x = ParseDate( $x );
            $y = ParseDate( $y );
            return $dir * ( $x cmp $y );
        },
        6 => sub {                                                           # Perceived Severity
            my ( $x, $y, $dir ) = @_;
            my %remap = (
                critical      => 5,
                major         => 4 ,
                minor         => 3,
                warning       => 2,
                indeterminate => 1,
                cleared       => 0
            );
            return $dir * ( $remap{$x} <=> $remap{$y} );
        }
    };

    bless $this, $class;
    return $this ;

}

##################################################################################
#
# Function    : display_main
#
# Parameters  : none
#
# Description : display alarm viewer main page
#
# Returns     : html string
#
# Called By   : avmain.htm
#
##################################################################################
sub display_main {
    my ($this) = @_ ;

    my $html = "" ;

    $html .= qq[<head>
<meta http-equiv="REFRESH" CONTENT="$this->{MW_system}{REFRESHRATE};URL=$this->{MW_system}{CGIPROG}?NPRSYS=$this->{MW_system}{NPRSYS}&NPRTARGET=avmain.htm" TARGET="MAIN-WINDOW" />
<link rel="stylesheet" href="/css/style.css" type="text/css">
<script language="javascript" src="/javascript/captions.js"></script>
<script language="javascript" src="/javascript/preload.js"></script>
<script language="javascript" src="/javascript/jssetselect.js"></script>
</HEAD>
<BODY onload="MM_preloadImages(
    '/images/buttons/checkAllOver.gif',
    '/images/buttons/clearAllOver.gif',
    '/images/buttons/deleteOver.gif')">
];

    $html .= "<p align=left><strong>Refresh Rate = </strong> $this->{MW_system}{'REFRESHRATE'} <strong> seconds </strong></p>\n";

    # initialize and process alarms for deletion
    my @deleteAlarm = ();

    my $formdata_hash = $this->{MW_formdata} ;
    foreach $key (keys(%$formdata_hash)) {
        if($key =~ /[0-9]/) {
            push @deleteAlarm, $key;
        }
    }

    if (uc($this->{MW_system}{'USERNAME'}) eq "SYSTEM") {
	if(@deleteAlarm ne 0) {
		$html .= $this->delete_alarm(@deleteAlarm);
	}
    } 

    # main

    $html .= $this->get_preferences();

    $this->get_alarm();

    $this->filter_alarm();

    $this->sort_alarm();

    $this->field_alarm();

    if (uc($this->{MW_system}{'USERNAME'}) eq "SYSTEM") {
	$html .= $this->admin_print_alarm();
    } else {
	$html .= $this->print_alarm();
    }

    $html .= qq[</BODY>
];

    return $html ;
}

##################################################################################
#
# Function    : display_avpreferences
#
# Parameters  : none
#
# Description : display alarm viewer preferences page
#
# Returns     : html string
#
# Called By   : avpreferences.htm
#
##################################################################################
sub display_avpreferences {
    my ($this) = @_ ;

    my $html = "" ;

    $html .= qq[
<head>
<title>Preferences</title>
<LINK REL="stylesheet" HREF="/css/style.css" TYPE="text/css" />
<script language="javascript" src="/javascript/alarms.js"></script>
<script language="javascript" src="/javascript/preload.js"></script>
</head>
<body onload="createColorChooser();MM_preloadImages(
    '/images/buttons/chooseOver.gif',
    '/images/buttons/applyOver.gif')">
];

    $html .= $this->get_preferences();

    $html .= qq[
<form name="preferences" method="POST" target="MAIN-WINDOW" action="$this->{MW_system}{'CGIPROG'}">
<input type=hidden name=NPRSYS value=$this->{MW_system}{'NPRSYS'} />
<input type=hidden name=NPRTARGET value="avmain.htm" />
<table border="1" class="tableblueboxes">
<tr><th colspan="2">Alarm Color</th></tr>
<tr>
    <td><input type="image" name="critical" src="/images/buttons/choose.gif"
        onmouseover="MM_swapImage('critical','','/images/buttons/chooseOver.gif',1)"
        onmouseout="MM_swapImgRestore()"
        onclick="chooseColor( event, 'critical');return false" /></td>
    <td class="critical">Critical</td>
</tr>
<tr>
    <td><input type="image" name="major" src="/images/buttons/choose.gif"
        onmouseover="MM_swapImage('major','','/images/buttons/chooseOver.gif',1)"
        onmouseout="MM_swapImgRestore()"
        onclick="chooseColor( event, 'major');return false" /></td>
    <td class="major">Major</td>
</tr>
<tr>
    <td><input type="image" name="minor" src="/images/buttons/choose.gif"
        onmouseover="MM_swapImage('minor','','/images/buttons/chooseOver.gif',1)"
        onmouseout="MM_swapImgRestore()"
        onclick="chooseColor( event, 'minor');return false" /></td>
    <td class="minor">Minor</td>
</tr>
<tr>
    <td><input type="image" name="warning" src="/images/buttons/choose.gif"
        onmouseover="MM_swapImage('warning','','/images/buttons/chooseOver.gif',1)"
        onmouseout="MM_swapImgRestore()"
        onclick="chooseColor( event, 'warning');return false" /></td>
    <td class="warning">Warning</td>
</tr>
<tr>
    <td><input type="image" name="indeterminate" src="/images/buttons/choose.gif"
        onmouseover="MM_swapImage('indeterminate','','/images/buttons/chooseOver.gif',1)"
        onmouseout="MM_swapImgRestore()"
        onclick="chooseColor( event, 'indeterminate');return false" /></td>
    <td class="indeterminate">Indeterminate</td>
</tr>
<tr>
    <td><input type="image" name="cleared" src="/images/buttons/choose.gif"
        onmouseover="MM_swapImage('cleared','','/images/buttons/chooseOver.gif',1)"
        onmouseout="MM_swapImgRestore()"
        onclick="chooseColor( event, 'cleared');return false" /></td>
    <td class="cleared">Cleared</td>
</tr>
<tr><th colspan="2">Refresh Rate</th></tr>
<tr><td colspan="2"><input type="text" name="REFRESHRATE" id="REFRESHRATE" value="$this->{MW_system}{'REFRESHRATE'}"></td></tr>
<!-- hidden value to transport preferences back to main page -->
<input type="hidden" name="PREFERENCEA" id="PREFERENCEA" value="$this->{MW_system}{'PREFERENCEA'}" />
<tr>
    <td colspan="2" align="center"><input type="image" name="apply" src="/images/buttons/apply.gif"
        onclick="updateMainWindow();return false" />
    </td>
</tr>
</table>
</form>

</body>
];

    return $html ;
}

##################################################################################
#
# Function    : display_avfilter
#
# Parameters  : none
#
# Description : display alarm viewer filter page
#
# Returns     : html string
#
# Called By   : avfilter.htm
#
##################################################################################
sub display_avfilter {
    my ($this) = @_ ;

    my $html = "" ;

    $html .= qq[
<head>
<title>Filter</title>
<link rel="stylesheet" href="/css/style.css" type="text/css" />
<script language="javascript" src="/javascript/preload.js"></script>
<script language="javascript" src="/javascript/avfilter.js"></script>
<script language="javascript">
<!--
_FILTERA = "$this->{MW_system}{'FILTERA'}";
_AGEA = "$this->{MW_system}{'AGEA'}";
//-->
</script>
</head>
<body onload="loadSettings();MM_preloadImages('/images/buttons/applyOver.gif')">

<form name="filter" method="POST" target="MAIN-WINDOW" action="$this->{MW_system}{'CGIPROG'}">
<input type="hidden" name="filterAge">
<input type="hidden" name="NPRSYS" value="$this->{MW_system}{'NPRSYS'}">
<input type="hidden" name="NPRTARGET" value="avmain.htm">
<input type="hidden" name="filterOrder">

<table border="1" align="center" class="tableblueboxes">
<tr align="center">
    <td>&nbsp;</td>
    <td>Show</td>
    <td>&nbsp;</td>
    <td>Hide</td>
</tr>
<tr align="center">
    <td>Severity</td>
    <td><select id="showSeverity" size="5" style="width: 150px" ondblclick="hideField( 'Severity' )"></td>
    <td style="width: 50px">
        <img border=0 src="/images/buttons/rightarrows.gif" onclick="hideField( 'Severity' )">
        <br>
        <br>
        <img border=0 src="/images/buttons/leftarrows.gif" onclick="showField( 'Severity' )">
    </td>
    <td><select id="hideSeverity" size="5" style="width: 150px" ondblclick="showField( 'Severity' )"></td>
</tr>
<tr align="center">
    <td>Instance</td>
    <td><select id="showInstance" size="5" style="width: 150px" ondblclick="hideField( 'Instance' )"></td>
    <td style="width: 50px">
        <img border=0 src="/images/buttons/rightarrows.gif" onclick="hideField( 'Instance' )">
        <br>
        <br>
        <img border=0 src="/images/buttons/leftarrows.gif" onclick="showField( 'Instance' )">
    </td>
    <td><select id="hideInstance" size="5" style="width: 150px" ondblclick="showField( 'Instance' )"></td>
</tr>
<tr align="center">
    <td>Class</td>
    <td><select id="showClass" size="5" style="width: 150px" ondblclick="hideField( 'Class' )"></td>
    <td style="width: 50px">
        <img border=0 src="/images/buttons/rightarrows.gif" onclick="hideField( 'Class' )">
        <br>
        <br>
        <img border=0 src="/images/buttons/leftarrows.gif" onclick="showField( 'Class' )">
    </td>
    <td><select id="hideClass" size="5" style="width: 150px" ondblclick="showField( 'Class' )"></td>
</tr>
<tr align="center">
    <td>Specific Problem</td>
    <td><select id="showProblem" size="5" style="width: 150px" ondblclick="hideField( 'Problem' )"></td>
    <td style="width: 50px">
        <img border=0 src="/images/buttons/rightarrows.gif" onclick="hideField( 'Problem' )">
        <br>
        <br>
        <img border=0 src="/images/buttons/leftarrows.gif" onclick="showField( 'Problem' )">
    </td>
    <td><select id="hideProblem" size="5" style="width: 150px" ondblclick="showField( 'Problem' )"></td>
</tr>
<tr align="center">
    <td>Event Type</td>
    <td><select id="showType" size="5" style="width: 150px" ondblclick="hideField( 'Type' )"></td>
    <td style="width: 50px">
        <img border=0 src="/images/buttons/rightarrows.gif" onclick="hideField( 'Type' )">
        <br>
        <br>
        <img border=0 src="/images/buttons/leftarrows.gif" onclick="showField( 'Type' )">
    </td>
    <td><select id="hideType" size="5" style="width: 150px" ondblclick="showField( 'Type' )"></td>
</tr>
<tr align="center">
    <td>Age (HH:MM)</td>
    <td>&nbsp;</td>
    <td>&nbsp;</td>
    <td>
        <input type="text" maxlength="2" size="2" style="text-align: right" name="filterAgeHours">&nbsp;:&nbsp;
        <input type="text" maxlength="2" size="2" style="text-align: right" name="filterAgeMinutes">
    </td>
</tr>
<tr align="center">
    <td>&nbsp;</td>
    <td>&nbsp;</td>
    <td style="width: 50px">
        <img src="/images/buttons/leftarrows.gif" style="border: 1px solid red" onclick="showAll()">
    </td>
    <td><input type="image" name="SUBMIT" src="/images/buttons/apply.gif"
        onmouseover="MM_swapImage('SUBMIT','','/images/buttons/applyOver.gif',1)"
        onmouseout="MM_swapImgRestore()"
        onclick="submitFilter();return false">
</tr>
</body>

</form>
];

    # populate select elements above
    $html .= $this->get_field();

    return $html ;
}

##################################################################################
#
# Function    : display_avfields
#
# Parameters  : none
#
# Description : display alarm viewer avfields page
#
# Returns     : html string
#
# Called By   : avfields.htm
#
##################################################################################
sub display_avfields {
    my ($this) = @_ ;

    my $html = "" ;

    $html .= qq[
<head>
<title>Fields</title>
<link rel="stylesheet" href="/css/style.css" type="text/css">
<script language="javascript" src="/javascript/preload.js"></script>
<script language="javascript" src="/javascript/avfields.js"></script>
<script language="javascript">
<!--
_FIELDA = "$this->{MW_system}{'FIELDA'}";
//-->
</script>
</head>
<body onload="loadSettings();MM_preloadImages(
    '/images/buttons/promoteOver.gif',
    '/images/buttons/demoteOver.gif',
    '/images/buttons/applyOver.gif')">
<form name="fields" method="POST" target="MAIN-WINDOW" action="$this->{MW_system}{'CGIPROG'}">
<input type=hidden name="NPRSYS" value="$this->{MW_system}{'NPRSYS'}">
<input type=hidden name="NPRTARGET" value="avmain.htm">
<input type="hidden" name="fieldsOrder">
<table border="1" align="center" class="tableblueboxes">
    <tr align="center">
        <td>Show</td>
        <td>&nbsp;</td>
        <td>Hide</td>
    </tr>
    <tr align="center">
        <td><select id="show" size="15" style="width: 150px" ondblclick="hideField()"></td>
        <td style="width: 50px">
            <img border=0 src="/images/buttons/rightarrows.gif" onclick="hideField()">
            <br>
            <br>
            <img border=0 src="/images/buttons/leftarrows.gif" onclick="showField()">
        </td>
        <td><select id="hide" size="15" style="width: 150px" ondblclick="showField()"></td>
    </tr>
    <tr>
        <td colspan="3" align="center">
        <input type="image" name="promote" src="/images/buttons/promote.gif"
            onmouseover="MM_swapImage('promote','','/images/buttons/promoteOver.gif',1)"
            onmouseout="MM_swapImgRestore()"
            onclick="promoteField();return false;">
        <input type="image" name="demote" src="/images/buttons/demote.gif"
            onmouseover="MM_swapImage('demote','','/images/buttons/demoteOver.gif',1)"
            onmouseout="MM_swapImgRestore()"
            onclick="demoteField();return false;">
        <input type="image" name="apply" src="/images/buttons/apply.gif"
            onmouseover="MM_swapImage('apply','','/images/buttons/applyOver.gif',1)"
            onmouseout="MM_swapImgRestore()"
            onclick="submitFields();return false;">
        </td>
    </tr>
</table>
</form>
</body>
];

    return $html ;
}

##################################################################################
#
# Function    : display_avsort
#
# Parameters  : none
#
# Description : display alarm viewer avsort page
#
# Returns     : html string
#
# Called By   : avsort.htm
#
##################################################################################
sub display_avsort {
    my ($this) = @_ ;

    my $html = "" ;

    $html .= qq[
<head>
<title>Sort</title>
<link rel="stylesheet" href="/css/style.css" type="text/css">
<script language="javascript" src="/javascript/preload.js"></script>
<script language="javascript" src="/javascript/avsort.js"></script>
<script language="javascript">
<!--
_SORTA="$this->{MW_system}{'SORTA'}";
//-->
</script>
</head>
<body onLoad="loadSettings();MM_preloadImages(
    '/images/buttons/promoteOver.gif',
    '/images/buttons/demoteOver.gif',
    '/images/buttons/toggleOver.gif',
    '/images/buttons/applyOver.gif')">
<!--create interface display-->
<form name="sort" method="POST" target="MAIN-WINDOW" action="$this->{MW_system}{'CGIPROG'}">
<input type=hidden name=NPRSYS value="$this->{MW_system}{'NPRSYS'}">
<input type=hidden name=NPRTARGET value="avmain.htm">
<input type="hidden" name=sortOrder>
<table border="1" align="center" class="tableblueboxes">
    <tr align="center">
        <td>Options</td>
        <td>&nbsp;</td>
        <td>Sort</td>
    </tr>
    <tr>
        <td><select id="hide" size="4" style="width: 220px" ondblclick="showField()"></td>
        <td style="width: 50px" align="center">
            <img border=0 src="/images/buttons/rightarrows.gif" onclick="showField()">
            <br>
            <br>
            <img border=0 src="/images/buttons/leftarrows.gif" onclick="hideField()">
        </td>
        <td><select id="show" size="4" style="width: 220px" ondblclick="hideField()"></td>
    </tr>
    <tr>
        <td colspan="3" align="center">
            <input type="image" name="promote" src="/images/buttons/promote.gif"
                onmouseover="MM_swapImage('promote','','/images/buttons/promoteOver.gif',1)"
                onmouseout="MM_swapImgRestore()"
                onclick="promoteField();return false">
            <input type="image" name="demote" src="/images/buttons/demote.gif"
                onmouseover="MM_swapImage('demote','','/images/buttons/demoteOver.gif',1)"
                onmouseout="MM_swapImgRestore()"
                onclick="demoteField();return false">
            <input type="image" name="toggle" src="/images/buttons/toggle.gif"
                onmouseover="MM_swapImage('toggle','','/images/buttons/toggleOver.gif',1)"
                onmouseout="MM_swapImgRestore()"
                onclick="toggleField();return false">
            <input type="image" name="apply" src="/images/buttons/apply.gif"
                onmouseover="MM_swapImage('apply','','/images/buttons/applyOver.gif',1)"
                onmouseout="MM_swapImgRestore()"
                onclick="submitSort();return false">
        </td>
    </tr>
</table>
</form>
</body>
];

    return $html ;
}

##################################################################################
#
# Function    : display_avmenu
#
# Parameters  : none
#
# Description : display alarm viewer avmenu page
#
# Returns     : html string
#
# Called By   : avmenu.htm
#
##################################################################################
sub display_avmenu {
    my ($this) = @_ ;

    my $html = "" ;

    $html .= qq[
<head>
    <meta http-equiv="REFRESH" CONTENT="$this->{MW_system}{'REFRESHRATE'};URL=$this->{MW_system}{'CGIPROG'}?NPRSYS=$this->{MW_system}{'NPRSYS'}&NPRTARGET=avmenu.htm" TARGET="MENU-WINDOW">
    <link rel="stylesheet" href="/css/style.css" type="text/css">

    <!-- override some styles -->
    <link rel="stylesheet" href="/css/header.css" type="text/css">
</head>
<body background="/images/topbg.gif" marginheight="0" marginwidth="0" topmargin="0" leftmargin="0">
<table width="100%" cellspacing="0" cellpadding="0" border="0">
    <tr>
        <td valign="top" width="250"><img src="/images/topgraphic1on.gif" width="246" height="56" border="0"></td>

        <td align="right">
            <table border="0">
                <tr>

                    <!-- Alarm status icon -->
                    <td colspan="4" align="right">
];
    $html .= $this->status_icon();
    $html .= qq[
                    </td>
                </tr>
                <tr>
                    <td><a class="header" href="javascript:alarmSortWin=window.open('$this->{MW_system}{'CGIPROG'}?NPRSYS=$this->{MW_system}{'NPRSYS'}&NPRTARGET=avsort.htm', 'alarmSort_$this->{MW_system}{'NPRSYS'}', 'width=570,height=150,resizable');alarmSortWin.focus()">&#149; sort</a></td>
                    <td><a class="header" href="javascript:alarmFilterWin=window.open('$this->{MW_system}{'CGIPROG'}?NPRSYS=$this->{MW_system}{'NPRSYS'}&NPRTARGET=avfilter.htm', 'alarmFilter_$this->{MW_system}{'NPRSYS'}', 'width=520,height=570,resizable');alarmFilterWin.focus()">&#149; filter</a></td>
                    <td><a class="header" href="javascript:alarmFieldsWin=window.open('$this->{MW_system}{'CGIPROG'}?NPRSYS=$this->{MW_system}{'NPRSYS'}&NPRTARGET=avfields.htm', 'alarmFields_$this->{MW_system}{'NPRSYS'}', 'width=420,height=370,resizable');alarmFieldsWin.focus()">&#149; fields</a></td>
                    <td><a class="header" href="javascript:alarmPreferencesWin=window.open('$this->{MW_system}{'CGIPROG'}?NPRSYS=$this->{MW_system}{'NPRSYS'}&NPRTARGET=avpreferences.htm', 'alarmPreferences_$this->{MW_system}{'NPRSYS'}', 'width=220,height=320,resizable');alarmPreferencesWin.focus()">&#149; preferences</a></td>
                </tr>
            </table>
        </td>
    </tr>
</table>
</body>
];

    return $html ;
}

##################################################################################
#
# Function    : status_icon
#
# Parameters  : none
#
# Description : display appropriate status icon
#
# Returns     : html string
#
# Called By   : menu
#
##################################################################################
sub status_icon {
    my ($this) = @_ ;
    $logger->log("info - IN: status_icon() ",4);

    my $bad  = qq{<img title="Alarm listener disabled" src="/images/status/disconnected.gif">\n};
    my $good = qq{<img title="Alarm listener enabled" src="/images/status/connected.gif">\n};

    my $pasInfo = $this->read_pas_info( $this->{MW_system}{'MW_NPRWEB_NPR_HOME'} );

    $logger->log("info - OUT: status_icon() ",4);

    return $bad unless exists $pasInfo->{'PARENTPID'};
    return $bad unless kill( 0 => $pasInfo->{'PARENTPID'} );
    return $bad unless kill( 0 => $pasInfo->{'CHILDPID'} );

    return $good;
}

##################################################################################
#
# Function    : get_preferences
#
# Parameters  : none
#
# Description : sets the colour output of alarms for severity
#
# Returns     : html string
#
# Called By   : main
#
##################################################################################
sub get_preferences {
    my ($this) = @_ ;
    $logger->log("info - IN: get_preferences() ",4);

    $logger->log( "Alarm colours string: $this->{MW_system}{'PREFERENCEA'}", 3);

    my %severityColor = split( /,/, $this->{MW_system}{'PREFERENCEA'} );

    my $html = "<style>\n" .
        join( "\n", map { ".$_ { color: white; background-color: $severityColor{$_} }" } sort keys %severityColor ) .
        "\n</style>\n";

    $logger->log("info - OUT: get_preferences() ",4);

    return $html ;
}

##################################################################################
#
# Function    : get_alarm
#
# Parameters  : none
#
# Description : loads the alarm data into an array
#
# Returns     : none
#
# Called By   : main
#
##################################################################################
sub get_alarm {
    my ($this) = @_ ;
    $logger->log("info - IN: get_alarm() ",4);

    # check for the existence of alarm files
    if (! -e $this->{alarmfile}) {
        $logger->log( "Could not find $this->{alarmfile}", 1 );
        return 0;
    }

    # open alarm file
    sysopen ALARM_FILE, $this->{alarmfile}, O_RDONLY 
        or $logger->log( "Could not open $this->{alarmfile}",1);

    # read alarms from file
    my @alarmFile = <ALARM_FILE>;
    $logger->log("info - alarmFile: $#alarmFile" ,4);
    $this->{MW_system}{displayAlarm} = \@alarmFile;

    close(ALARM_FILE);
    
    # Count the number of alarms in ALARM_FILE
    $this->{MW_system}{initNumAlarms} = scalar @alarmFile;
    $logger->log("info - initNumAlarms: $this->{MW_system}{initNumAlarms}" ,4);
    $logger->log("info - OUT: get_alarm() ",4);

}

##################################################################################
#
# Function    : print_alarm
#
# Parameters  : none
#
# Description : prints the alarm data
#
# Returns     : none
#
# Called By   : main
#
##################################################################################
sub print_alarm {
    my ($this) = @_ ;
    $logger->log("info - IN: print_alarm() ",4);

my($alarmLine, @alarm);
my $alarmHeader = $this->{MW_system}{alarmHeader};
my $html = "";

# global variable
%tableHeader = (
    0 => "Object Class",
    1 => "Object Name",
    2 => "Event Type",
    3 => "Event Time",
    4 => "Probable Cause",
    5 => "Specific Problem",
    6 => "Perceived Severity",
    7 => "Trend Indication",
    8 => "Threshold Info",
    9 => "Alarm Id",
    10 => "Alarm Correlated Ids",
    11 => "Monitored Attribute",
    12 => "Attribute Value",
    13 => "Alarm Text",
    14 => "Dispatch Time"
);

    # initialize table
    $html .= "<TABLE ALIGN=LEFT WIDTH=100% BORDER=1 class=tableblueboxes>\n";
    # print table header
    for $i (0 .. $#$alarmHeader) {  #get alarm_header values
        # print alarm in tabular form
        $html .= "<TD NOWRAP> $tableHeader{$alarmHeader->[$i]} </TD>";
    }


    # Count the number of alarms in $this->{MW_system}{displayAlarm}
    my $alarmsList = $this->{MW_system}{displayAlarm} ;
    my $finalNumAlarms = scalar @$alarmsList;

    # print list
    for $alarmLine (@$alarmsList) {
        #split alarm values
        @alarm = split(/;/, $alarmLine);
        $html .= qq{<TR class="$alarm[6]">};
        # print alarm values
        for $i (0 .. $#$alarmHeader) { #get alarm_header values
            if($alarm[$alarmHeader->[$i]] eq "") {
                $html .= "<TD NOWRAP>. </TD>";
            } else {
                $html .= "<TD NOWRAP> $alarm[$alarmHeader->[$i]] </TD>";
            }
        }
        $html .= "</TR>";
    }
    $html .= "\n</TABLE>\n";

    $html .= "\n<br clear=left><br>\n<p><strong>Displaying</strong> $finalNumAlarms <strong>of</strong> $this->{MW_system}{initNumAlarms} <strong>alarms</strong></p>\n";

    $logger->log("info - OUT: print_alarm() ",4);

    return $html ;
}

##################################################################################
#
# Function    : admin_print_alarm
#
# Parameters  : none
#
# Description : prints the alarm data + an extra field to clear the alarms
#
# Returns     : html string
#
# Called By   : main
#
##################################################################################
sub admin_print_alarm {
    my ($this) = @_ ;
    $logger->log("info - IN: admin_print_alarm() ",4);
    my $alarmHeader = $this->{MW_system}{alarmHeader} ;
    my $html = "" ;

my($alarmLine, @alarm);
my %adminTableHeader = (
    0 => "Object Class",
    1 => "Object Name",
    2 => "Event Type",
    3 => "Event Time",
    4 => "Probable Cause",
    5 => "Specific Problem",
    6 => "Perceived Severity",
    7 => "Trend Indication",
    8 => "Threshold Info",
    9 => "Alarm Id",
    10 => "Alarm Correlated Ids",
    11 => "Monitored Attribute",
    12 => "Attribute Value",
    13 => "Alarm Text",
    14 => "Dispatch Time"

);

    # set checkboxes
    $html .= qq{
        <script>
            function setCheck( truth ) {
                var inputs = document.getElementsByTagName( 'input' );
                for( var i = 0; i < inputs.length ; i++ ) {
                    if( inputs[i].getAttribute( 'type' ).match( /checkbox/i ) ||
                        inputs[i].getAttribute( 'TYPE' ).match( /checkbox/i ) ) {
                            inputs[i].checked = truth;
                    }
                }
            }
        </script>
    };

    # initialize form and table
    $html .= "<FORM name='admin' method=post action=$this->{MW_system}{'CGIPROG'}>\n"; 
    $html .= "<input type=hidden name=NPRTARGET value='avmain.htm'>\n";
    $html .= "<input type=hidden name=NPRSYS value=$this->{MW_system}{'NPRSYS'}>\n";
    $html .= "<input type=hidden name='Alarm Id' value=$adminTableHeader{$alarm[10]}>\n";
    $html .= "<TABLE ALIGN=LEFT WIDTH=100% BORDER=1 class=tableblueboxes>\n";

    # print table header
    $html .= "<TD NOWRAP>&nbsp;</TD>";
    for $i (0 .. $#$alarmHeader) {  #get alarm_header values
        # print alarm in tabular form
        $html .= "<TD NOWRAP> $adminTableHeader{$alarmHeader->[$i]} </TD>";
    }

    # Count the number of alarms displayAlarm
    my $alarmsList = $this->{MW_system}{displayAlarm} ;
    my $finalNumAlarms = scalar @$alarmsList;

    # print list
    for $alarmLine (@$alarmsList) {
        #split alarm values
        @alarm = split(/;/, $alarmLine);
        $html .= qq{<TR class="$alarm[6]">};
        $html .= "<TD NOWRAP><INPUT TYPE=CHECKBOX NAME=$alarm[9]></TD>\n"; 
        # print alarm values
        for $i (0 .. $#$alarmHeader) { #get alarm_header values
            if($alarm[$alarmHeader->[$i]] eq "") {
                $html .= "<TD NOWRAP>. </TD>";
            } else {
                $html .= "<TD NOWRAP> $alarm[$alarmHeader->[$i]] </TD>";
            }
        }
        $html .= "</TR>\n";
    }
    $html .= "<tr>\n";
    $html .= "<td colspan=99>";
    $html .= qq{
        <input type='image' name='checkAll' src='/images/buttons/checkAll.gif'
            onmouseover="MM_swapImage('checkAll','','/images/buttons/checkAllOver.gif',1)"
            onmouseout="MM_swapImgRestore()"
            onclick='setCheck( true );return false'>
        <input type='image' name='clearAll' src='/images/buttons/clearAll.gif'
            onmouseover="MM_swapImage('clearAll','','/images/buttons/clearAllOver.gif',1)"
            onmouseout="MM_swapImgRestore()"
            onclick='setCheck( false );return false'>
        <input type='image' name='delete' src='/images/buttons/delete.gif'
            onmouseover="MM_swapImage('delete','','/images/buttons/deleteOver.gif',1)"
            onmouseout="MM_swapImgRestore()"
            onclick='admin.submit();return false'>
    };
    $html .= "</tr>\n";
    $html .= "</TABLE>\n";
    $html .= "\n<br clear=left><br>\n<p><strong>Displaying</strong> $finalNumAlarms <strong>of</strong> $this->{MW_system}{initNumAlarms} <strong>alarms</strong></p>\n";
    $html .= "</FORM>\n";

    $logger->log("info - OUT: admin_print_alarm() ",4);

    return $html ;
}

##################################################################################
#
# Function    : delete_alarm
#
# Parameters  : none
#
# Description : For new listener/writer, the writer keeps an in memory linked list
#               of alarms and refreshes the avalarms file on a periodic basis.
#               Therefore, the old method of delete_alarms updating the same file
#               will no longer work. Instead, we write the alarm ids to a 
#               temporary file, and signal the writer process to read this
#               file to update its in memory copy.
#
# Returns     : html string
#
# Called By   : main
#
##################################################################################
sub delete_alarm {
    my( $this, @alarms ) = @_;
    $logger->log("info - IN: delete_alarm() ",4);

    my $deleteFile = "$this->{alarmfile}.delete.$$";

    open ( DELETE, ">>$deleteFile" ) or $logger->log( "Could not open $deleteFile", 1 );

    print DELETE join( "\n", @alarms ), "\n";

    close DELETE;

    # find the writer process
    my $pasInfo = $this->read_pas_info( $this->{MW_system}{'MW_NPRWEB_NPR_HOME'} );
    if ( ! $pasInfo->{'CHILDPID'} ) {
        $logger->log("info - delete_alarm: Alarm processes file could not be found. Alarms cannot be deleted.",2);
        $html .= qq[<script language="JavaScript">
            alert( "Alarm processes file could not be found.\\nAlarms cannot be deleted.");
        </script>];
        return;
    }
    # check the writer process
    if( ! kill 0 => $pasInfo->{'CHILDPID'} ) {
        $logger->log("info - delete_alarm: Alarm writer process could not be found. Alarms cannot be deleted.",2);
        $html .= qq[<script language="JavaScript">
            alert( "Alarm writer process could not be found.\\nAlarms cannot be deleted.");
        </script>];
        return;
    }

    # now signal the writer to update its in memory alarms queue
    kill( USR1 => $pasInfo->{'CHILDPID'} );

    # the writer should remove the file once finished
    # make sure we don't get into an infinite loop
    for( my $loop = 0; $loop < 30; $loop++ ) {
        sleep(1);
        return unless -f $deleteFile ;
    }

    # should not reach here
    $logger->log("info - delete_alarm: Timed out waiting for alarms to be deleted.",2);
    $html .= qq[<script language="JavaScript">
        alert( "Timed out waiting for alarms to be deleted.\\nPlease refresh page.");
    </script>];

    $logger->log("info - OUT: delete_alarm() ",4);
    return $html ;
}

##################################################################################
#
# Function    : filter_alarm
#
# Parameters  : none
#
# Description : filters the alarms
#
# Returns     : none
#
# Called By   : main
#
##################################################################################
sub filter_alarm {
    my ($this) = @_ ;
    $logger->log("info - IN: filter_alarm() ",4);

    my(%filterOrder);
    my(@alarm, $tempAlarm);
    my @newAlarm = () ;

    $this->filter_age();

    # Getting the filter order from the system variables
    %filterOrder = split(/,/,$this->{MW_system}{'FILTERA'});
    $logger->log( "Alarm filter string: $this->{MW_system}{'FILTERA'}", 3);

    # Taking a local copy of the alarms to be processed
    $tempAlarm = $this->{MW_system}{displayAlarm} ;
    $this->{MW_system}{displayAlarm} = [];

    my ($t_alarm, $b_alarm);
    # Loop through each alarm to see if it should be filtered out
    ALARM: foreach $t_alarm ( @$tempAlarm ) {

        @alarm = split(/;/,$t_alarm); 
        
        # Loop over each field of the alarm to see if the
        # alarm should be displayed.  If one of the fields
        # indicates that the alarm should not be displayed
        # then start the next alarm
        foreach $b_alarm ( @alarm ) {
            next ALARM if( exists($filterOrder{$b_alarm})
                and ( ' ' eq $filterOrder{$b_alarm}) );
        }
        
        # Add this alarm to the list of alarms that should be
        # displayed.
           push @newAlarm, $t_alarm;
    }
    $this->{MW_system}{displayAlarm} = [ @newAlarm ];

    $logger->log("info - OUT: filter_alarm() ",4);
}

##################################################################################
#
# Function    : filter_age
#
# Parameters  : none
#
# Description : filters the alarms where age >= specified seconds
#
# Returns     : html string
#
# Called By   : filter_alarm
#
##################################################################################
sub filter_age {
    my ($this) = @_ ;
    $logger->log("info - IN: filter_age() ",4);

    my (@alarm, $age);
    my ($tempAlarm, $alarmSeconds, $secondsGroup);
    my $newAlarm = () ;

    # store data
    $age = $this->{MW_system}{'AGEA'};
    $logger->log( "Alarm age filter setting: $this->{MW_system}{'AGEA'}", 3);
   
    if ($age > 0) {
        $secondsGroup = &DateCalc("$age seconds ago","today",\$err);

        $logger->log( "\$secondsGroup = $secondsGroup", 4 );
        $tempAlarm = $this->{MW_system}{displayAlarm} ;
   
        for $j (0..$#$tempAlarm) {
            @alarm = split(/;/,$tempAlarm->[$j]);
            $alarmSeconds = ParseDate($alarm[3]);
            $logger->log( "\$alarmSeconds = $alarmSeconds for alarm ID $alarm[9]", 4 );
            if ($alarmSeconds gt $secondsGroup) {
                push @newAlarm, $tempAlarm->[$j];
            }    
        }

        $this->{MW_system}{displayAlarm} = [ @newAlarm ];
    }
    $logger->log("info - OUT: filter_age() ",4);
}

##################################################################################
#
# Function    : get_field
#
# Parameters  : none
#
# Description : initialization for the filter dialog box and default settings
#
# Returns     : html string
#
# Called By   : avfilter.htm
#
##################################################################################
sub get_field {
    my ($this) = @_ ;
    $logger->log("info - IN: get_field() ",4);

    my %selectBox = (
        0 => "showClass",
        1 => "showInstance",
        2 => "showType",
        5 => "showProblem",
        6 => "showSeverity",
    );

    # unique occurences
    my %unique;

    # open alarms file 
    # use simple retry mechanism
    sysopen ALARM_FILE, $this->{alarmfile}, O_RDONLY 
        or $logger->log( "Could not open $this->{alarmfile}",1);

    while( <ALARM_FILE> ) {
        chomp;
        my @field = split /;/;
        map { $unique{$_}->{$field[$_]}++ } keys %selectBox;
    }
    close ALARM_FILE;

    # generate javascript to populate boxes accordingly
    $html .= qq{<script language="javascript">};
    foreach my $select ( keys %selectBox ) {
        $html .= qq{
            var select = document.getElementById( "$selectBox{$select}" );
            select.options.length = 0;
        };
        foreach ( sort keys %{$unique{$select}} ) {
            $html .= qq{ select.options[ select.options.length ] = new Option( "$_" )\n};
        }
    }
    $html .= qq{</script>};

    $logger->log("info - OUT: get_field() ",4);

    return $html ;
}

##################################################################################
#
# Function    : sort_alarm
#
# Parameters  : none
#
# Description : sort alarms
#
# Returns     : none
#
# Called By   : main
#
##################################################################################
sub sort_alarm {
    my ($this) = @_ ;
    $logger->log("info - IN: sort_alarm() ",4);

    use CGI qw( :all );
    use CGI::Carp qw( fatalsToBrowser );

    # looks like this is needed to make ParseDate behave?
    $ea_temp = ParseDate("08:41 06-Sep-95");

    $logger->log( "Alarm sort string: $this->{MW_system}{'SORTA'}", 3);
    my @sortSetting = split( /,/, $this->{MW_system}{'SORTA'}, -1 );
    my @sortOrder = ();
    my $alarmsList = $this->{MW_system}{displayAlarm} ;


    for( my $i = 0; $i <= $#sortSetting; $i += 3 ) {

        my $type      = $sortSetting[ $i ];
        my $direction = $sortSetting[ $i + 1 ];
        my $flag      = $sortSetting[ $i + 2 ];

        $direction = ( $direction =~ /ASC/ ? 1 : -1 );

        if( $flag eq '*' ) {
            push @sortOrder, { type => $type, direction => $direction };
        }
    }

    @$alarmsList = sort {
        @a = split( /;/, $a, -1 );
        @b = split( /;/, $b, -1 );
        foreach ( @sortOrder ) {
            my $type      = $_->{'type'};
            my $direction = $_->{'direction'};

            $aa = $a[ $type ];
            $bb = $b[ $type ];
            my $ret = &{$this->{sortType}{ $type }}( $aa, $bb, $direction );

            if( $ret != 0 ) { return $ret; }
        }
        return 0;
    }  @$alarmsList;
    $this->{MW_system}{displayAlarm} = $alarmsList ;
    $logger->log("info - OUT: sort_alarm() ",4);
}

##################################################################################
#
# Function    : field_alarm
#
# Parameters  : none
#
# Description : filters the header fields
#
# Returns     : none
#
# Called By   : main
#
##################################################################################
sub field_alarm {
    my ($this) = @_ ;
    $logger->log("info - IN: field_alarm() ",4);

    my(@fieldOrder) = ();

    # store data
    push @fieldOrder, split(/,/,$this->{MW_system}{'FIELDA'});
    $logger->log( "Alarm fields string: $this->{MW_system}{'FIELDA'}", 3);

    @alarmHeader = () ;

    for $i (0..$#fieldOrder) {
        if ($fieldOrder[$i] eq '*') { 
            push @alarmHeader, $fieldOrder[$i-1];
        }
    }
    $this->{MW_system}{alarmHeader} = \@alarmHeader ;
    $logger->log("info - OUT: field_alarm() ",4);
}

##################################################################################
#
# Function    : read_pas_info
#
# Parameters  : $dir -- the full path of the NPR directory under the
#               webserver directory that this PAS listener is
#               associated with
#
# Description : Reads the info from the file webserver/NPR?/tmp/listener_pids
#               and places it into a hash
#
# Returns     : A reference to a hash that contains two entries, PARENTPID and
#               CHILDPID.  If file is found or it contains nothing a
#               reference to an empty hash is returned.
#
# Called By   : 
#
##################################################################################
sub read_pas_info {
    my ($this, $dir)=@_;
    $logger->log("info - IN: read_pas_info() ",4);
    my $psinfo={};
    if(open(PSINFO,"<$dir/tmp/listener_pids")) {
                # There should only be one line the file.  It contains the two
                # process identifiers of the listener and its child.
        my $line = <PSINFO>;
        chomp $line;
        my ($parentPid,$kidPid)=split /:/, $line;
        $psinfo->{'PARENTPID'}=$parentPid;
        $psinfo->{'CHILDPID'}=$kidPid;
        close PSINFO;
    }
    $logger->log("info - OUT: read_pas_info() ",4);
    return $psinfo;
}


1;
