# ************************************************************
# @(#) web/webserver/cgi/Core/Form/ConfigServer.pm web_r3.6.0.8:cset.001111:22:33@(#)
#
# Filename    : web/webserver/cgi/Core/Form/ConfigServer.pm
#
# *************************************************************
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                       Vallent
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# Copyright (c) 2005 Vallent Software Systems UK. All rights reserved.
#
# **************************************************************
#
#   Description :
#
# ************************************************************
#
package Core::Form::ConfigServer;

require Core::Form::KeyUtils;

@ISA = qw{ Core::Form::KeyUtils };

use CGI qw(:standard);
use File::stat;

# set up logging
require "$ENV{'NPRWEB_DIR'}/cgi/Lib/Logger.pm";
my $logger = new Logger( $ENV{NPRWEB_NPR_LOGFILE}, $ENV{NPRWEB_NPR_LOGLEVEL} );

##################################################################################
#
# Function    : new()
#
# Parameters  : MW_system - hash of form variables,
#
# Description : Constructor for server configuration
#
# Returns     : object reference
#
# Called By   : ConfigServer.htm
#
##################################################################################
sub new {
    my $proto = shift;    # get the class name
    my $class = ref($proto) || $proto;
    # if class exists, use it else use reference.
    my $this = {MW_system => {},
                NPR_WWW_LOGFILE => '',
                @_} ;

    bless $this, $class;
    return $this ;
}

##################################################################################
#
# Function    : main()
#
# Parameters  : none
#
# Description : Object method to construct the Configure Metrica/Web Servers page
#
# Returns     : html string
#
# Called By   : ConfigServer.htm
#
##################################################################################
sub main {

    my ($this) = @_ ;
    my $output = "";

    # Flag to indicate whether to write out html tags with messages
    my $html=0;
    $html=1 if(exists $ENV{GATEWAY_INTERFACE});

    if($html) {
	$output .= qq[
	<head>
	<title>Configure Metrica/Web Servers</title>
	<LINK REL="stylesheet" HREF="/css/style.css" TYPE="text/css">
	</head>
	<script language="javascript" src="/javascript/preload.js"></script>
	<script language=javascript>
	// Function to show the 'Please wait' animation
	var runAnimwait = false;
	function animwait() {
	    runAnimwait = true;

	    // Clear the <pre> message if there is any
	    if ( document.getElementById("message") ) {
		var m = document.getElementById("message");
		m.style.display = 'none';
	    }

	    // Create <div> tag to show 'Please wait ...'
	    waitingDiv = document.createElement("div");
	    waitingDiv.setAttribute("id", "waiting");
	    waitingDiv.setAttribute("align", "center");

	    // Create <p> tag to hold the waiting text
	    msg = document.createTextNode("Processing request, please wait ...");
	    waitingText = document.createElement("p") ;
	    //waitingText.style.fontSize = "20px" ;
	    waitingText.appendChild( msg );
	    waitingDiv.appendChild( waitingText );

	    // Define the image file
	    //waitingImg = document.createElement("img") ;
	    //waitingImg.setAttribute("src", "/images/animwait.gif");
	    //waitingDiv.appendChild( waitingImg );

	    // Append the text and image to the 'nprform'
	    document.nprform.appendChild( waitingDiv );	
	}

	// Function to process start/restart/stop/refresh request
	function newaction(req_val){
	    if ( !runAnimwait ) {
		animwait();
	    }
	    document.nprform.request.value = req_val;
	}
	</script>
	<body class=large; onload="MM_preloadImages('/images/buttons/ResetOver.gif','/images/buttons/startAllServersOver.gif','/images/buttons/restartAllServersOver.gif','/images/buttons/stopAllServersOver.gif','/images/buttons/refreshScreenOver.gif','/images/buttons/ListenOver.gif','/images/buttons/IgnoreOver.gif')">
	<p align=center class="bluehead0">Configure Metrica/Web Servers</p>
	];
    }

    # Update footer to include link to Current Licence Key Info and Current User Info
    #
    #Remove licence information for bug 56873
    #    <p><a href="#" onclick=window.open('$this->{MW_system}{CGIPROG}?NPRSYS=$this->{MW_system}{NPRSYS}&NPRTARGET=LicenceInfo.htm','licence','height=300,width=400')>Current licence key information</a></p>
    my $footer=qq[
    <p><a href="#" onclick=window.open('$this->{MW_system}{CGIPROG}?NPRSYS=$this->{MW_system}{NPRSYS}&NPRTARGET=CurrentUser.htm','user_$this->{MW_system}{NPRSYS}','height=400,width=400,scrollbars')>Current User information</a></p></body>
    ];

    my $allocfile = "$this->{MW_system}{MW_NPRWEB_HOME}/licensing/alloc";
    my $alloc=$this->read_allocation($allocfile);
    $alloc={} unless($alloc);

    # Read in the 'nprinfo' file which gives the names and locations of all of the databases
    my $nprinfofile = "$this->{MW_system}{MW_NPRWEB_HOME}/nprinfo";
    my $dblist=$this->read_db_info($nprinfofile);

    # Messages will appear under the second <hr> on the web page so we'll stash
    # them in $message
    my $message;

    # Some temporary variables to reduce clutter, $dbref is a hash reference, $db the name
    # of an NPR installation directory within the webserver e.g. NPR1
    my ($dbref,$db);

    # Go through the alloc for current NPR system and setting it to integer of what the
    # user has typed this has the beneficial side effect that text becomes 0
    # Also set any null values in the allocation hash to 0
    foreach $dbref (@$dblist) {
	if($dbref->{NAME} eq $this->{MW_system}{NPRSYS}) {
	    my $name=$dbref->{NAME};

	    param($name,int(param($name)));
	    $alloc->{$name}=0 unless($alloc->{$name});
	}
    }

    # Set a flag to say if we're here simply to refresh the information or (implicitly)
    # to perform an action
    my $action='none';
    if($html) {
	if(param('request')){
	    $action = param('request');
	}
    }

    my $cmd;
    # Test for being in the CGI environment as we don't want to update allocation from the
    # shell prompt because this will result in null entries in the alloc file and thus
    # no processes starting
    if($html and $action ne 'none' and $action ne 'refresh') {
	# update the allocation hash and write it out to disk
	foreach $dbref (@$dblist) { 
	    if($dbref->{NAME} eq $this->{MW_system}{NPRSYS}) {
		$alloc->{$dbref->{NAME}}=param($dbref->{NAME});
	    }
	}

	unless( $this->write_allocation($allocfile,$alloc) ) {
            $message="Unable to write to allocation file ($allocfile)\n";
	} else {
            $message="Licence allocation file updated ($allocfile)\n";
	}
    }

    # Now go through all the NPR systems (read from nprinfo) and check for the 
    # current NPR system that you have accessed to
    # Stop and then start the NPR system with the newly allocated values
    foreach $dbref (@$dblist) {
	if($dbref->{NAME} eq $this->{MW_system}{NPRSYS}) {
	    $dbref->{PATH} =~ s/\/$//;  # remove any trailing slash
	    if($action eq 'stop' or $action eq 'restart') {
		$cmd="$dbref->{PATH}/bin/nprwwwhelper $dbref->{PATH} nprwwwstop $this->{MW_system}{MW_NPRWEB_HOME}/$dbref->{NAME}/webconfig.sh";
		$message .= "\nStopping '$dbref->{DESC}'\n";
		$message .= $this->runcmd($cmd);
	    }
	    if($action eq 'start' or $action eq 'restart') {
		if($alloc->{$dbref->{NAME}} > 0) {
		    $cmd="$dbref->{PATH}/bin/nprwwwhelper $dbref->{PATH} nprwwwstart $alloc->{$dbref->{NAME}} $this->{MW_system}{MW_NPRWEB_HOME}/$dbref->{NAME}/webconfig.sh";
		    $message .= "\nStarting '$dbref->{DESC}'\n";
		    $message .= $this->runcmd($cmd);
		}
	    }
	    unless($action eq 'refresh') {
		if($dbref->{PASHOST} and $dbref->{PASPORT}) {
		    if( (param("pas$dbref->{NAME}start") eq 'Listen') or ($action eq 'start' and $html==0)) {
			$message.="Starting PAS Listener for '$dbref->{DESC}'\n";
			$this->start_listener($dbref,\$message,$this->{MW_system}{MW_NPRWEB_HOME});

                    } elsif( (param("pas$dbref->{NAME}stop") eq 'Ignore' ) or ($action eq 'stop' and $html==0)) {
			$message.="Stopping PAS Listener for '$dbref->{DESC}'\n";
			$this->stop_listener($dbref,\$message,$this->{MW_system}{MW_NPRWEB_HOME});
		    }
		}
            }
	}
    }

    if($html) {
	# Now display the table with the current allocation
	$output .= "<form action=\"$this->{MW_system}{CGIPROG}\" method=post name=nprform>\n";

	# Set a default hidden field to use when selecting the action
	$output .= "<input type='hidden' name='request' value='none'>\n";
	$output .= "<input type='hidden' name='NPRSYS' value='$this->{MW_system}{NPRSYS}'>\n";
	$output .= "<input type='hidden' name='NPRTARGET' value='ConfigServer.htm'>\n";

	$output .= qq[
<table cellpadding=3>
  <tr><td class='tableblueboxes'>NPR system</td><td class='tableblueboxes'>Instances</td>
      <td class='tableblueboxes'>Location</td><td class='tableblueboxes'>Status</td>\n];

	foreach $dbref (@$dblist) {
	    if($dbref->{PASHOST} and $dbref->{PASPORT}) {
		$output .= "      <td class='tableblueboxes' colspan='2'>PAS Listener</td>\n";
		last;
	    }
	}	
	$output .= "  </tr>\n";
	my $status;
	
	foreach $dbref (@$dblist) {
	    # Display only the current NPR system that you have accessed to
	    if($dbref->{NAME} eq $this->{MW_system}{NPRSYS}) {
		# set up javascript functions to populate alarm input elements for use
		# when starting and stopping listener
		$output .= qq[
    <script language=javascript>
    function alarmListen$dbref->{NAME}() {
	if ( !runAnimwait ) {
	    animwait();
	}
 	document.nprform.pas$dbref->{NAME}start.value='Listen';
    }
    function alarmIgnore$dbref->{NAME}() {
	if ( !runAnimwait ) {
	    animwait();
	}
   	document.nprform.pas$dbref->{NAME}stop.value='Ignore';
    }
    </script>
  <tr><td class="tableblueboxes1">$dbref->{DESC}</td>
    <td class="tableblueboxes1"><input size=5 type=text name=$dbref->{NAME} value=$alloc->{$dbref->{NAME}}></td>
    <td class="tableblueboxes1">$dbref->{PATH}</td>];

		$cmd="$dbref->{PATH}/bin/nprwwwhelper $dbref->{PATH} nprwwwstate $alloc->{$dbref->{NAME}}";
		$status = $this->runcmd($cmd);
		chomp $status;
		$output .= "<td class='tableblueboxes1'>$status</td>\n";

        	if($dbref->{PASHOST} and $dbref->{PASPORT}) {
	            # Create dummy input fields to be populated on submition
	            # by javascript alarmListen() or alarmIgnore();
		    $output .= qq[
    <td class='tableblueboxes1'>
    <input type=hidden name=pas$dbref->{NAME}start value='none'>
    <input type=hidden name=pas$dbref->{NAME}stop value='none'>];

		    $output .= qq[
    <input type=image src='/images/buttons/Listen.gif' title='Click to start alarm Listener' id="$dbref->{NAME}listen" 
	onmouseover="MM_swapImage('$dbref->{NAME}listen','','/images/buttons/ListenOver.gif',1)" 
	onmouseout="MM_swapImgRestore()" onclick='alarmListen$dbref->{NAME}();'><br>
    <input type=image src='/images/buttons/Ignore.gif' title='Click to stop alarm Listener' id="$dbref->{NAME}ignore"
	onmouseover="MM_swapImage('$dbref->{NAME}ignore','','/images/buttons/IgnoreOver.gif',1)"
	onmouseout="MM_swapImgRestore()" onclick='alarmIgnore$dbref->{NAME}();'>\n];

		    my $listen_status = $this->check_listener($dbref,\$message,$this->{MW_system}{MW_NPRWEB_HOME});
		    if($listen_status == 1) { 
			$output .= "    <td class='tableblueboxes1'>Listening</td>\n"
		    } else { 
			$output .= "    <td class='tableblueboxes1'>Stopped</td>\n"
		    }
		}
	    }
	}

	$output .= qq[
  </tr></table>
    <input type=image id=Reset src='/images/buttons/Reset.gif' onmouseover="MM_swapImage('Reset','','/images/buttons/ResetOver.gif',1)" onmouseout="MM_swapImgRestore()" onclick='reset(); return false;'>

    <p><input type=image id=start src='/images/buttons/startAllServers.gif' title='Click to start all servers' 
	onmouseover="MM_swapImage('start','','/images/buttons/startAllServersOver.gif',1)" 
	onmouseout="MM_swapImgRestore()" onclick='newaction("start");'>
      <input type=image id=restart src='/images/buttons/restartAllServers.gif' title='Click to restart all servers' 
	onmouseover="MM_swapImage('restart','','/images/buttons/restartAllServersOver.gif',1)" 
	onmouseout="MM_swapImgRestore()" onclick='newaction("restart");'>
      <input type=image id=stop src='/images/buttons/stopAllServers.gif' title='Click to stop all servers' 
	onmouseover="MM_swapImage('stop','','/images/buttons/stopAllServersOver.gif',1)" 
	onmouseout="MM_swapImgRestore()" onclick='newaction("stop");'>
      <input type=image id=refresh src='/images/buttons/refreshScreen.gif' title='Click to refresh screen' 
	onmouseover="MM_swapImage('refresh','','/images/buttons/refreshScreenOver.gif',1)" 
	onmouseout="MM_swapImgRestore()" onclick='newaction("refresh");'>
</form>
<hr>];

	if( $message ) { 
	    $output .= "<p><b><pre id='message'>$message</pre></b><hr>\n";
	}

	# Refresh the frames/page accordingly after the start/stop action
	#
	$output .= qq[
    <script language=javascript>
	var url_string = "$ENV{URL}";
	var result = url_string.match(/(request=start)|(request=restart)/);
	var result2 = url_string.match(/(request=stop)/);
	if(result != null) {
	    reloadFrame('header');
	    reloadFrame('TOC');
	}
	if(result2 != null) {
	    reloadFrame('header');
	}
    </script>];

	$output .= $footer;
    } else {
	$output .= $message;
    }
    return $output;

} # End of main() function


################################################################################
# Function	:  licence_info()
#
# Parameters	:  None
#
# Description	:  Function to display the current licence key settings
#
# Returns	:  html string
#
# Called By   	:  main()
#
##################################################################################
sub licence_info {
    my ($this) = @_;
    my ($lic_total, $lic_inuse, $cmd);
    my $output = "";
    my $html = 0;

    $html = 1 if (exists $ENV{GATEWAY_INTERFACE});
    if ($html) {
	$output .= qq[
	<head>
	<title>Current Metrica/Web Licensing</title>
	<LINK REL="stylesheet" HREF="/css/style.css" TYPE="text/css">
	</head>
	<body background="/images/framebg.gif">
	];
    }

    # we need to know the NPR specific PATH
    # and since the nprlicnum is not database specific just pick the first
    # database installation.
    #
    # what is needed to is the top level directory where the nprlicnum
    # script resides and this is a Metrica/WEB specific version, not
    # the standard one.
    #
    $ENV{PREFIX} = "$this->{MW_system}{MW_NPRWEB_HOME}/bin";

    $cmd = "$this->{MW_system}{MW_NPRWEB_HOME}/bin/nprlicnum -owned -print ADC_WEB_TRUNK";
    $lic_total  = $this->runcmd( $cmd );
    $cmd = "$this->{MW_system}{MW_NPRWEB_HOME}/bin/nprlicnum -inuse -print ADC_WEB_TRUNK";
    $lic_inuse  = $this->runcmd( $cmd );

    if (not defined ($lic_total)) {
	$lic_total = 0;
    }
    if (not defined ($lic_inuse)) {
	$lic_inuse = 0;
    }

    if ($html) {
	$output .= qq[
	<table width="300" border="0" cellpadding="1" cellspacing="0" align="center">
	<tr>
	<td class="titlebar" align="center" height="30">
	    <p class="bluehead1">Licence Key Information</p>
	</td>
	</tr>
	</table>
	<table border=0 align="center">
	    <th align=left class="tableblueboxes">&nbsp;In use&nbsp;</th>
	    <th align=left class="tableblueboxes">&nbsp;Total&nbsp;</th>
	    <tr class="tableblueboxes1">
	    <td>&nbsp;$lic_inuse</td>
	    <td>&nbsp;$lic_total</td>
	    </tr>
	</table>
	<a href="#" onclick=window.close()>Close</a>
	</body>
	</html>
	];
    }
    return $output;
}


################################################################################
# Function	:  current_user()
#
# Parameters	:  None
#
# Description	:  Function to display the current users logon to Metrica/Web
#
# Returns	:  html string
#
# Called By   	:  main()
#
##################################################################################
sub current_user {
    my ($this) = @_;
    my ($file,$title,$element);
    my (@line);
    my $output = "";
    my $html = 0;

    $html = 1 if (exists $ENV{GATEWAY_INTERFACE});
    if ($html) {
	$output .= qq[
	<head>
	<title>Current Metrica/Web Users</title>
	<LINK REL="stylesheet" HREF="/css/style.css" TYPE="text/css">
	</head>
	<body background="/images/framebg.gif">
	<table width="300" border="0" cellpadding="1" cellspacing="0" align="center">
	    <tr><td class="titlebar" align="center" height="30">
	    <p class="bluehead1">User Information</p>
	</td></tr>
	</table>
	];
    }

    # run the script that produces the script
    my $nprsys = "$this->{MW_system}{NPRSYS}";
    my $file="$this->{MW_system}{MW_NPRWEB_NPR_HOME}/data/output";
    $this->get_users($file,$nprsys);

    # create the start of the table
    $output .= "<table border='1' align='center'>\n";
    open (OUTFILE, "< $file");
    my $l_count = 0;
    while(<OUTFILE>){
	chomp;
	#how many lines in file ?
	next if ((/^$/) or (/^-+$/));
	if($l_count == 0){
            #table header
            $l_count++;
            $output .= "<tr>\n";

            my @line = split (/\s+/, $_);
            #There are space at the beginnig of the line
            #This creates an empty first element which
            # shold be removed
            if(!$line[0]){shift @line}
            foreach $title (@line) {
                $output .= "<td class='tableblueboxes'>$title</td>\n";
            }
            $output .= "</tr>\n";
        } elsif(/^There.*/) {
            #final line
            $output .= "<tr>\n";
            $output .= "<td class='tableblueboxes' colspan='4'>$_</td>\n";
            $output .= "</tr>\n";
        } else {
            # Normal table data
            $output .= "<tr>\n";
            # The rest are normal lines
            my @line = split (/\s+/, $_);

            #There are space at the beginnig of the line
            #This creates an empty first element which
            # shold be removed
            if(!$line[0]){shift @line}
            foreach $element (@line) {
                $output .= "<td class='tableblueboxes1'>$element</td>\n";
            }
            $output .= "</tr>\n";
        }
    }
    $output .= "</table>\n";
    $output .= "<a href='#' onclick='window.close()'>Close</a>\n";
    $output .= "</body></html>";

    unlink($file);

    return $output;
}


################################################################################
# Function	:  get_users()
#
# Parameters	:  $out_file -- Temporary output file to display current user
#				info
#		   $nprsys -- the NPR system that web connected to
#
# Description	:  Function to read-in current user info from .dat file and 
#		   display the output in html format
#
# Returns	:  None	  
#
# Called By   	:  current_user()
#
##################################################################################
sub get_users {
    my ($this, $out_file, $nprsys) = @_;
    my (@dirs);
    my ($npr_dir, $npr, $dud, $dir, $file_name, $file_path, $st, $f_handle, $type, $name);

    # if an output file is defined open it
    if($out_file){
	open (OUTFILE, "> $out_file");
    }

    # Get the NPR directories from nprinfo
    #open (NPRINFO, "$this->{MW_system}{MW_NPRWEB_HOME}/nprinfo");
    #while ( <NPRINFO> ) {
	#chomp;
	#($npr_dir, $dud) = split(':',$_,2);
	#push (@dirs, $npr_dir);
    #}
    #close(NPRINFO);

    # if outfile is defined make sure print statement go to that file
    if ($out_file){
	$f_handle = select(OUTFILE);
    }

    # Initialise count of users
    my $nusers = 0;

	$dir = "$this->{MW_system}{MW_NPRWEB_NPR_HOME}/data";
	opendir(DATA, $dir) || die ("0; Cannot open $dir");
	    print ("\n\n\n");
	    printf ("%6s %10s %10s %6s\n", "USER", "SYSTEM", "FILE", "IDLE");
	    print "---------------------------------------\n";

	    # Go through the files in $dir
	    while ( defined ($file_name = readdir DATA) ) {
		if($file_name =~ /.dat/){
		    $file_path = "$dir/$file_name";
		    open (FILE, $file_path) || die "0; Cannot open file $file_path";
		    $st = stat($file_path) || die " unable to stat file $file_path";

		    # Calculate the idle time of the file in hours and minutes
		    my $time_now = time();
		    my $atime = ($time_now - $st->ctime);
		    my ($hour, $min, $rem);
		    $hour = $atime/3600;
		    $rem = $atime%3600;
		    $min = $rem/60;

		    # Get the USERNAME from the session file and print the
		    # filename username and idle time.
		    while( <FILE> ) {
			chomp;
			next if /^$/;
			($type, $name) = split /:/;
			if ( $type =~ /^USERNAME/ ) {
			    #increment the number of users
			    $nusers++;
			    printf ("%10s %6s %10s \t%02d:%02d \n", $name, $nprsys, $file_name, $hour, $min);
			}
		    }
		    close(FILE);
		}
	    }

	    print "---------------------------------------\n";
	    if ($nusers == 1){
		print "There is $nusers user currently logged on to Metrica/Web\n\n";
	    } else {
		print "There are $nusers users currently logged on to Metrica/Web\n\n";
	    }
    closedir(DATA);

    if($out_file){
	# return output to STDOUT
	select($f_handle);
	# close the file
	close(OUTFILE);
    }
    return 0;
}



################################################################################
# Function	:  runcmd()
#
# Parameters	:  $cmd -- the command to execute
#
# Description	:  Run a command, putting the stdout and stderr into a scalar
#                  that it returns
#
# Returns	:  Scalar giving the output of the command executed
#
# Called By   	:  main()
#
##################################################################################
sub runcmd {
    my ($this, $cmd) = @_;
    my $res;

    if(open(STAT,"$cmd 2>&1 |")) {
        while(<STAT>) { $res.=" $_" }
        if( not close STAT ) {
			# Checking that it is not errno 10, this was a
			# problem on RS6000. Any other problems report an
			# error saying the process was not successfully closed.
			if( 10 != $! ) {
				$res.=" Cannot close process for '$cmd' $?\n";
			}
		}
    }
    else { $res.="  Cannot run '$cmd'\n" }
    return $res;
}


1;
