# *************************************************************
# @(#) web/webserver/cgi/Lib/nprcookie.pl web_r3.6.0.8:cset.001111:22:33@(#)
#
#   Filename    : ../cgi/Lib/nprcookie.pl
#
# *************************************************************
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                       Vallent
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# Copyright (c) 2005 Vallent Software Systems UK. All rights reserved.
#
# **************************************************************
#
#	Description :	nprcookie.pl responsible to set and read from HTTP cookies.
#								
#
# ************************************************************

use Data::Dumper;

###############################################################################
###	Following are the default values used when settin the cookie.
###			$CookieExpiry	: Optional cookie's expiration date 
###			$CookieDomain	: Optional domain for which cookie is valid
###			$CookiePath		: Optional path for which cookie is valid
###			$CookieSecure	: if set to 1, the cookie will only be used within secure session
###
###			%decode_char  : hash containing characters conversion
###############################################################################
$CookieExpiry='';
$CookieDomain='';
$CookiePath='';
$CookieSecure=0;

my %decode_char = (
	'%26'	=> "\&",
	'%3A'	=> "\:"
);


###############################################################################
###	Description	: Read from the $ENV{'HTTP_COOKIE'}
###############################################################################
sub NPRGetCookies {
    local($cookie,$value,$httpcookie);
		
    # Start process HTTP_COOKIES
    $httpcookie = $ENV{'HTTP_COOKIE'};
    if ( ! $httpcookie ) {
			return 0;
		};
				
		# Get the cookie name and value and make sure it's readable
		#		HTTP_COOKIE => NPRLOGON=USERNAME%3A%3Asystem%26PASSWORD%3A%3Asy5GOK8n6AmCA%26STATEID%3A%3A1407
		#	NPRLOGON=USERNAME%3A%3Asystem%26PASSWORD%3A%3Asy5GOK8n6AmCA%26STATEID%3A%3A23229; DRG1=%7C
		foreach my $nowcookie (split(/; /,$ENV{'HTTP_COOKIE'})) {
			($cookie,$value) = split(/=/, $nowcookie);
			foreach $char (keys %decode_char) {
				$cookie =~ s/$char/$decode_char{$char}/g;
				$value =~ s/$char/$decode_char{$char}/g;
		  };
		  
		  # Process each cookie elements
		  #		$value = USERNAME%3A%3Asystem%26PASSWORD%3A%3Asy5GOK8n6AmCA%26STATEID%3A%3A1407
		  # and put into hash
		  my $cookie_nprsys;
		  foreach ( split(/&/, $value) ) {
			  ($cookie,$value) = split(/::/);
			  if ($cookie eq 'NPRSYS') {
			  	$cookie_nprsys = $value;
			  }
			 $cookie_hash{$cookie_nprsys}{$cookie} = $value; 
		  };
	  }
	  
	  return 1;
};

###############################################################################
###	Description	: Set the $ENV{'HTTP_COOKIE'}
###	Input	: Cookie's name and array of cookies-values
###############################################################################
sub NPRSetCookies {
    local($cookiename,%cookiehash) = @_;
    local($cookievalue);

    my $index=0;
    foreach my $csysname (keys %cookiehash) {
	if ($index>0) {
		$cookievalue .= '%26' . 'NPRSYS%3A%3A' . $csysname;
	}else{
		$cookievalue = 'NPRSYS%3A%3A' . $csysname;
	}
	foreach my $syscookie (keys %{$cookiehash{$csysname}}) {
		$cookievalue .= '%26' . $syscookie . '%3A%3A' . $cookiehash{$csysname}{$syscookie};
	}
	$index++;
     }

    # Start writing the cookies
    print 'Set-Cookie: ' . $cookiename . '=' . $cookievalue . ';';
		if ($CookieExpiry) {
			print ' expires='.$CookieExpiry.';';
		}
		
		if ($CookiePath) {
			print ' path=' . $CookiePath . ';';
		}

		# If a domain has been set, add it to the header.
		if ($CookieDomain) {
			print ' domain='.$CookieDomain.';';
		}

		# If this cookie should be sent only over secure channels, add that  #
		# to the header.                                                     #
		if ($CookieSecure) {
			print ' secure';
		}

		# End this line of the header, setting the cookie.                   #
		print "\n";
};

###############################################################################
###	Description	: Set the cookie expiration date
###								If set, will write to cookie as 
###										.. expires=Tue, 20-Mac-2007 15:30:00 GMT
###	Input : number of seconds before expire
###					define as '0' will not set the Expires 
###					and the cookie will remain active until the user quits the browser
###############################################################################
sub NPRSetCookieExpiry() {
  my ($seconds) = @_;
  if ($seconds < 1 || !$seconds) {
	  return;
  };

  my $sydate=gmtime(time+$seconds);
  my ($day, $month, $num, $time, $year) = split(/\s+/,$sydate);
  $num = sprintf("%02d", $num);

  $CookieExpiry="$day, $num-$month-$year $time GMT";

  return;
};

###############################################################################
###	Description	: if set the browser will return the cookie to any host 
###								that matches the partial domain name
###								eg; defining as "vallent.com" will let browser return the
###										cookies to any host, running on any machine ended with ".vallent.com"
###############################################################################
sub NPRSetCookieDomain() {
  my ($domain) = @_;
  if (!$domain) {
	  return;
  };

  $CookieDomain=$domain;

  return;
};

###############################################################################
###	Description	: If set, the cookie will only be sent to your script 
###								if the CGI request is occurring on a secure channel, such as SSL
###############################################################################
sub NPRSetCookieSecure() {
  my ($secure) = @_;
  
  if ($secure) {
	  $CookieSecure=1;
  };
  
  return;
};

###############################################################################
###	Description	: If set, the browser will check it against your script's 
###								URL before returning the cookie. Example, when defined as '/cgi'
###								the cookie will be returned to each of the scripts within /cgi only.
###								It take '/' as default where cookie to be sent to any CGI script on your site
###############################################################################
sub NPRSetCookiePath() {
  my ($path) = @_;
  
  if (!$path) {
	  return;
  };
  
  $CookiePath=$path;

  return;
};

1;
